"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@nowa/core");
const execa_1 = require("execa");
class ModuleScript extends core_1.Module.Async {
    constructor() {
        super(...arguments);
        this.$name = 'script';
        this.alreadyRun = false;
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { logger } = this.$utils;
            const [scripts, options] = this.$runtime.config;
            this.options = options || {};
            this.scripts = [].concat(scripts).map(this.validateScript);
            logger.info(`got ${this.scripts.length} scripts`);
            logger.debug(this.scripts);
        });
    }
    run() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { logger } = this.$utils;
            if (this.alreadyRun && this.options && this.options.noRetrigger) {
                logger.debug('skip run scripts since noRetrigger has been set');
            }
            else {
                logger.debug('start to run scripts');
                if (this.options && this.options.noWait) {
                    logger.debug('noWait mode');
                    this._run().then(() => logger.debug('finish all scripts'), e => {
                        logger.error(e);
                        process.exit(1);
                    });
                }
                else {
                    logger.debug('normal mode');
                    yield this._run();
                }
                logger.debug('finish all scripts');
                this.alreadyRun = true;
            }
        });
    }
    _run() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { logger } = this.$utils;
            if (this.options && this.options.parallel) {
                logger.debug('parallel mode');
                yield Promise.all(this.scripts.map(this._runScript, this));
            }
            else {
                logger.debug('sequential mode');
                for (const script of this.scripts) {
                    yield this._runScript(script);
                }
            }
        });
    }
    _runScript(script) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { logger } = this.$utils;
            if (typeof script === 'string') {
                logger.info('run shell', script);
                yield execa_1.shell(script, { stdio: 'inherit', cwd: this.$runtime.context, maxBuffer: 100000000 });
            }
            else if (typeof script === 'function') {
                logger.info('run js', script.name || 'anonymous');
                yield script();
            }
        });
    }
    validateScript(script) {
        if (typeof script === 'string' || typeof script === 'function') {
            return script;
        }
        const { logger } = this.$utils;
        logger.error(`script should be string / function but received ${typeof script}`);
        throw new Error('invalid typeof script');
    }
}
exports.default = ModuleScript;
